<?php
/**
 * Created by PhpStorm.
 * User: tsecher
 * Date: 12/10/2018
 * Time: 17:38
 */

namespace Drupal\form_helper_widget\Services;

use Cocur\Slugify\Slugify;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\Display\EntityFormDisplayInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\field_ui\Form\EntityDisplayFormBase;
use Drupal\form_helper_widget\Form\FormHelperWidgetAddForm;
use Psr\Log\InvalidArgumentException;

class FormHelperWidgetConf {
  const SERVICE_NAME = 'form_helper_widget.conf';

  const FIELD_ID = 'value';
  const FIELD_DESCRIPTION = 'description';
  const FIELD_FIRST_ID = 'first_id';
  const THIRD_PARTY_Id = 'form_helper_widget';

  /** @var \Cocur\Slugify\Slugify $slugify */
  protected $slugify;

  public function __construct() {
    $this->slugify = Slugify::create(['separator'=>'_']);
  }


  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Ajoute le widget dans la conf tierse.
   *
   * @param $group
   * @param null $display
   *
   * @return \Drupal\Core\Entity\EntityInterface|null|static
   */
  public function saveWidget($group, $display = NULL) {
    if ($display === NULL) {
      if ($group->context == 'form') {
        $display = EntityFormDisplay::load($group->entity_type . '.' . $group->bundle . '.' . $group->mode);
      }
    }

    // If no display was found. It doesn't exist yet, create it.
    if (!isset($display)) {
      if ($group->context == 'form') {
        $display = EntityFormDisplay::create(array(
          'targetEntityType' => $group->entity_type,
          'bundle'           => $group->bundle,
          'mode'             => $group->mode,
        ))->setStatus(TRUE);
      }
    }

    if (isset($display)) {
      $data = (array) $group;
      unset($data['entity_type'], $data['bundle'], $data['mode'], $data['form'], $data['context']);
      // On slugify l'id pour la gestion de ponctuation.
      $id = substr($this->slugify->slugify($data[static::FIELD_ID]), 0, 10) . rand(1, 100);
      $data[static::FIELD_FIRST_ID] = $id;
      $display->setThirdPartySetting(static::THIRD_PARTY_Id, $id, $data);
      $display->save();
    }

    return $display;
  }

  /**
   * Initialise le rendu du widget dans le formulaire d'édition.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $formState
   */
  public function showWidgetConf(array &$form, FormStateInterface $formState) {
    // Only start altering the form if we need to.
    if (empty($form['#fields']) && empty($form['#extra'])) {
      return;
    }

    $display = $this->getDisplayFromFormState($formState);
    $params = field_group_field_ui_form_params($form, $display);

    // Récupération des translation_widgets.
    $this->cleanThirdParty($params->entity_type, $params->bundle, $params->mode);
    $widgets = $display->getThirdPartySettings(static::THIRD_PARTY_Id);


    $formStateValues = $formState->getValues();

    foreach ($widgets as $id => $widget) {
      if( $id ){
        // Save the group when the configuration is submitted.
        if ($formState->getTriggeringElement()['#name'] == FormHelperWidgetManager::me()
            ->slugify($id) . '_plugin_settings_update') {
          $this->updateWidget($form, $formState, $id, $formStateValues['fields'][$id]['format']);
        }

        $this->addWidgetConfig($form['fields'], $id, $widget, $formState, $params, $display);
        $form['#' . static::THIRD_PARTY_Id][$id] = $widget;
      }
    }

    // Gestion de la soumission du formulaire.
    array_unshift($form['actions']['submit']['#submit'], get_called_class() . '::onConfigFormSubmit');
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $formState
   */
  public static function onConfigFormSubmit(array &$form, FormStateInterface $formState) {
    if (array_key_exists('#' . static::THIRD_PARTY_Id, $form)) {
      $fields = $formState->getValue('fields');
      foreach ($form['#' . static::THIRD_PARTY_Id] as $widgetName => $widgetData) {
        static::me()
          ->updateWidget($form, $formState, $widgetName, $fields[$widgetName]);
      }
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $formState
   * @param $widgetId
   * @param $formData
   */
  public function updateWidget(array &$form, FormStateInterface $formState, $widgetId, $formData) {
    $display = $this->getDisplayFromFormState($formState);
    $widgets = $display->getThirdPartySettings(static::THIRD_PARTY_Id);

    // On actualise les valeurs.
    if( array_key_exists($widgetId, $widgets) ){
      foreach ($widgets[$widgetId] as $key => $value) {
        if (array_key_exists($key, $formData)) {
          $widgets[$widgetId][$key] = $formData[$key];
        }
      }

      $widgets = array_filter($widgets);

      $display->setThirdPartySetting(static::THIRD_PARTY_Id, $widgetId, $widgets[$widgetId]);
      $display->save();
    }

  }

  /**
   * Récupération du display depuis le formulaire.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   * @return EntityFormDisplay;
   */
  protected function getDisplayFromFormState(FormStateInterface $formState) {
    $callback_object = $formState->getBuildInfo()['callback_object'];
    if (!$callback_object instanceof EntityDisplayFormBase) {
      throw new InvalidArgumentException('Unkown callback object.');
    }

    // Récupération du display.
    return $callback_object->getEntity();
  }

  /**
   * @param $table
   * @param $id
   * @param $widget
   * @param \Drupal\Core\Form\FormStateInterface $formState
   */
  protected function addWidgetConfig(&$table, $id, $widget, FormStateInterface $formState, $params, EntityFormDisplayInterface $display) {

    // A group cannot be selected as its own parent.
    $parent_options = $table['#parent_options'];
    unset($parent_options[$id]);

    $table[$id] = array(
      '#attributes'      => array('class' => array('draggable'), 'id' => $id),
      '#row_type'        => 'group',
      '#js_settings'     => array('rowHandler' => 'group'),
      '#region_callback' => get_called_class() . '::regionCallback',
      'human_name'       => array(
        '#markup' => Html::escape(t($widget[static::FIELD_ID])),
        '#prefix' => '<span class="group-label">',
        '#suffix' => '</span>',
      ),
      'weight'           => array(
        '#type'          => 'textfield',
        '#default_value' => $widget['weight'],
        '#size'          => 3,
        '#attributes'    => array('class' => array('field-weight')),
      ),
      'parent_wrapper'   => array(
        'parent'      => array(
          '#type'          => 'select',
          '#options'       => $parent_options,
          '#empty_value'   => '',
          '#default_value' => isset($params->parents[$id]) ? $params->parents[$id] : '',
          '#attributes'    => array('class' => array('field-parent')),
          '#parents'       => array('fields', $id, 'parent'),
        ),
        'hidden_name' => array(
          '#type'          => 'hidden',
          '#default_value' => $id,
          '#attributes'    => array('class' => array('field-name')),
        ),
      ),
    );

    $table[$id]['format'] = FormHelperWidgetAddForm::getFields($widget);

    $base_button = array(
      '#submit'     => array(
        array($formState->getBuildInfo()['callback_object'], 'multistepSubmit')
      ),
      '#ajax'       => array(
        'callback' => array(
          $formState->getBuildInfo()['callback_object'],
          'multistepAjax'
        ),
        'wrapper'  => 'field-display-overview-wrapper',
        'effect'   => 'fade',
      ),
      '#field_name' => $id,
    );

    if ($formState->get('plugin_settings_edit') == $id) {
      $table[$id]['format']['#cell_attributes'] = array('colspan' => 2);
      $table[$id]['format']['format_settings'] = array(
        '#type'       => 'container',
        '#attributes' => array('class' => array('field-plugin-settings-edit-form')),
        '#parents'    => array('fields', $id, 'settings_edit_form'),
        '#weight'     => -5,
        'label'       => array(
          '#markup' => t('Field group format:') . ' <span class="formatter-name">' . '</span>',
        ),
        // Create a settings form where hooks can pick in.
        'settings'    => [],
        'actions'     => array(
          '#type'           => 'actions',
          'save_settings'   => $base_button + array(
              '#type'  => 'submit',
              '#name'  => FormHelperWidgetManager::me()
                  ->slugify($id) . '_plugin_settings_update',
              '#value' => t('Update'),
              '#op'    => 'update',
            ),
          'cancel_settings' => $base_button + array(
              '#type'                    => 'submit',
              '#name'                    => $id . '_plugin_settings_cancel',
              '#value'                   => t('Cancel'),
              '#op'                      => 'cancel',
              // Do not check errors for the 'Cancel' button.
              '#limit_validation_errors' => array(),
            ),
        ),
      );
      $table[$id]['#attributes']['class'][] = 'field-formatter-settings-editing';
      $table[$id]['format']['type']['#attributes']['class'] = array('visually-hidden');
    }
    else {
      // After saving, the settings are updated here aswell. First we create
      // the element for the table cell.
      $table[$id]['settings_summary'] = array('#markup' => '');

      // Add the configure button.
      $table[$id]['settings_edit'] = $base_button + array(
          '#type'                    => 'image_button',
          '#name'                    => $id . '_group_settings_edit',
          '#src'                     => 'core/misc/icons/787878/cog.svg',
          '#attributes'              => array(
            'class' => array('field-plugin-settings-edit'),
            'alt'   => t('Edit')
          ),
          '#op'                      => 'edit',
          // Do not check errors for the 'Edit' button, but make sure we get
          // the value of the 'plugin type' select.
          '#limit_validation_errors' => array(array('fields', $id, 'type')),
          '#prefix'                  => '<div class="field-plugin-settings-edit-wrapper">',
          '#suffix'                  => '</div>',
        );

      if ($delete_route = $this->getDeleteRoute($id, $display)) {
        $table[$id]['settings_edit']['#suffix'] .= Link::fromTextAndUrl(t('delete'), $delete_route)->toString();
      }
    }
  }

  /**
   * @param $row
   * @return string
   */
  public static function regionCallback($row) {
    switch ($row['#row_type']) {
      case 'group':
        return 'content';
    }
  }

  /**
   * Retourne la route de suppression.
   *
   * @param $widgetId
   */
  public function getDeleteRoute($widgetId, EntityFormDisplayInterface $display) {
    $entityTypeId = $display->getTargetEntityTypeId();
    $entityType = \Drupal::entityTypeManager()->getDefinition($entityTypeId);

    if ($entityType->get('field_ui_base_route')) {

      $parameters = [
        'entityTypeId' => $entityTypeId,
        'mode'         => $display->getMode(),
        'bundle'       => $display->getTargetBundle(),
        'widgetId'     => $widgetId,
      ];

      $routeName = 'field_ui.form_helper_widget_delete_' . $entityType->id();
      return new Url($routeName, $parameters);
    }

    return NULL;
  }

  /**
   * @param $entityType
   * @param $bundle
   * @param $mode
   * @param $widgetId
   */
  public function deleteWidget($entityType, $bundle, $mode, $widgetId) {
    if ($display = EntityFormDisplay::load($entityType . '.' . $bundle . '.' . $mode)) {
      $display->unsetThirdPartySetting(static::THIRD_PARTY_Id, $widgetId);
      $display->save();

      \Drupal::messenger()
        ->addMessage(t('Le widget a correctement été supprimé'));
    }
    else {
      \Drupal::messenger()->addMessage(t('Une erreur s\'est produite'));
    }
  }

  /**
   * Clean les thirds party settings.
   * @param $entityType
   * @param $bundle
   * @param $mode
   */
  public function cleanThirdParty($entityType, $bundle, $mode) {
    if ($display = EntityFormDisplay::load($entityType . '.' . $bundle . '.' . $mode)) {
      $widgets = $display->getThirdPartySettings(static::THIRD_PARTY_Id);
      foreach ( $widgets as $widgetId => $widgetData){
        if( is_null($widgetId) || is_null($widgetData) ){
          $display->unsetThirdPartySetting(static::THIRD_PARTY_Id, $widgetId);
        }
      }

      $display->save();
    }
  }

}
