<?php

namespace Drupal\form_helper_widget\Services;


use Cocur\Slugify\Slugify;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Form\ConfirmFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\lmwr_tools\Service\LanguageService;
use Drupal\locale\SourceString;

class FormHelperWidgetManager {

  const SERVICE_NAME = 'form_helper_widget.manager';

  /** @var \Cocur\Slugify\Slugify $slugifier */
  protected $slugifier = NULL;

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Retourne le widget de traduction.
   *
   * @param array $form
   *   Le formulaire.
   * @param $stringId
   *   L'identifiant du string à traduire.
   * @param $widgetName
   *   Le nom affiché du widget.
   * @param null $description
   *   La description du widget.
   *
   * @return array
   */
  public function getWidget(array &$form, $stringId, $description = NULL) {

    $slug = $this->slugify($stringId);
    // On ajoute le widget à la liste des
    $form['#' . FormHelperWidgetConf::THIRD_PARTY_Id][] = [
      'slug'                         => $slug,
      FormHelperWidgetConf::FIELD_ID => $stringId
    ];


    return [
      '#theme'  => 'form_helper_widget',
      '#title'   => $stringId,
      '#content' => nl2br($description['value'])
    ];

    return $container;
  }

  /**
   * Slugify un string.
   *
   * @param string $string
   *
   * @return string
   */
  public function slugify($string) {
    if (is_null($this->slugifier)) {
      $this->slugifier = Slugify::create([]);
    }

    return $this->slugifier->slugify($string);
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $formState
   */
  public function showWidgetInForm(array &$form, FormStateInterface $formState) {
    $form_object = $formState->getFormObject();
    if ($form_object instanceof ContentEntityFormInterface && !$form_object instanceof ConfirmFormInterface) {

      /**
       * @var \Drupal\Core\Entity\Display\EntityFormDisplayInterface $form_display
       */
      $storage = $formState->getStorage();
      if (!empty($storage['form_display'])) {

        // Récupération des paramètres.
        $thirdPartyData = $storage['form_display']->getThirdPartySettings(FormHelperWidgetConf::THIRD_PARTY_Id);
        if (!empty($thirdPartyData)) {
          foreach ($thirdPartyData as $data) {
            $values = $this->getWidget($form, $data[FormHelperWidgetConf::FIELD_ID], $data[FormHelperWidgetConf::FIELD_DESCRIPTION]);
            $values['#weight'] = $data['weight'];
            $fieldId = empty($data[FormHelperWidgetConf::FIELD_FIRST_ID]) ? $data[FormHelperWidgetConf::FIELD_ID] : $data[FormHelperWidgetConf::FIELD_FIRST_ID];
            $form[$fieldId] = $values;
          }

          $form['actions']['submit']['#submit'][] = get_called_class() . '::onSubmit';
        }
      }
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $formState
   */
  public static function onSubmit(array $form, FormStateInterface $formState) {
    if (array_key_exists('#' . FormHelperWidgetConf::THIRD_PARTY_Id, $form)) {
      $service = static::me();
      foreach ($form['#' . FormHelperWidgetConf::THIRD_PARTY_Id] as $widgetData) {
        $service->saveTranslation($widgetData[FormHelperWidgetConf::FIELD_ID], $formState->getUserInput()[$widgetData['slug']], NULL, FALSE);
      }

      $service->flushTranslationCache();
    }
  }

}
